//StarCatV, Star Catalog Visualization Tool
// (C) 1999  Jim Campbell   jc@izzy.net
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


// Novas.h: interface for the Novas class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_NOVAS_H__2B3CE12F_5F0D_11D3_AB4E_444553540000__INCLUDED_)
#define AFX_NOVAS_H__2B3CE12F_5F0D_11D3_AB4E_444553540000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//From Org Novas.h
  #include <stdio.h>
  #include <math.h>
  #include <string.h>
  #include <stdlib.h>
  #include <ctype.h>



//From Org Novas.h
//Structures.
/*
   struct body: designates a celestial object.

   type              = type of body
                     = 0 ... major planet, Sun, or Moon
                     = 1 ... minor planet
   number            = body number
                       For 'type' = 0: Mercury = 1, ..., Pluto = 9,
                                       Sun = 10, Moon = 11
                       For 'type' = 1: minor planet number
   name              = name of the body (limited to 99 characters)
*/

   typedef struct
   {
      short int type;
      short int number;
      char name[100];
   } body;

/*
   struct site_info: data for the observer's location.  The atmospheric 
                     parameters are used only by the refraction 
                     function called from function 'equ_to_hor'.
                     Additional parameters can be added to this 
                     structure if a more sophisticated refraction model 
                     is employed.
                     
   latitude           = geodetic latitude in degrees; north positive.
   longitude          = geodetic longitude in degrees; east positive.
   height             = height of the observer in meters.
   temperature        = temperature (degrees Celsius).
   pressure           = atmospheric pressure (millibars)
*/

   typedef struct
   {
      double latitude;
      double longitude;
      double height;
      double temperature;
      double pressure;
   } site_info;

/*
   struct cat_entry: the astrometric catalog data for a star; equator 
                     and equinox and units will depend on the catalog.
                     While this structure can be used as a generic
                     container for catalog data, all high-level 
                     NOVAS-C functions require J2000.0 catalog data 
                     with FK5-type units (shown in square brackets
                     below).
                     
   catalog[4]         = 3-character catalog designator.
   starname[51]       = name of star.
   starnumber         = integer identifier assigned to star.
   ra                 = mean right ascension [hours].
   dec                = mean declination [degrees].
   promora            = proper motion in RA [seconds of time per 
                        century].
   promodec           = proper motion in declination [arcseconds per 
                        century].
   parallax           = parallax [arcseconds].
   radialvelocity     = radial velocity [kilometers per second].
*/

   typedef struct
   {
      char catalog[4];
      char starname[51];
      long int starnumber;
      double ra;
      double dec;
      double promora;
      double promodec;
      double parallax;
      double radialvelocity;
   } cat_entry;

//From Org Novas.c
//Define "origin" constants.
   #define BARYC  0
   #define HELIOC 1


class Novas  
{
public:
	Novas();
	virtual ~Novas();

   short int app_star (double tjd, body *earth, cat_entry *star,

                       double *ra, double *dec);

   short int app_planet (double tjd, body *ss_object, body *earth, 

                         double *ra, double *dec, double *dis);

   short int topo_star (double tjd, body *earth, double deltat,
                        cat_entry *star, site_info *location, 

                        double *ra, double *dec);

   short int topo_planet (double tjd, body *ss_object, body *earth,
                          double deltat, site_info *location, 

                          double *ra, double *dec, double *dis);

   short int virtual_star (double tjd, body *earth, cat_entry *star,

                           double *ra, double *dec);

   short int virtual_planet (double tjd, body *ss_object, body *earth,

                             double *ra, double *dec, double *dis);

   short int local_star (double tjd, body *earth, double deltat,
                         cat_entry *star, site_info *location,

                         double *ra, double *dec);

   short int local_planet (double tjd, body *ss_object, body *earth,
                           double deltat, site_info *location,

                           double *ra, double *dec, double *dis);

   short int astro_star (double tjd, body *earth, cat_entry *star,

                         double *ra, double *dec);

   short int astro_planet (double tjd, body *ss_object, body *earth,

                           double *ra, double *dec, double *dis);

   short int mean_star (double tjd, body *earth, double ra, double dec,

                        double *mra, double *mdec);

   void sidereal_time (double julianhi, double julianlo, double ee,

                       double *gst);

   void pnsw (double tjd, double gast, double x, double y, 
              double *vece,

              double *vecs);

   void spin (double st, double *pos1,

              double *pos2);

   void wobble (double x, double y, double *pos1,

               double *pos2);

   void terra (site_info *locale, double st,

               double *pos, double *vel);

   void earthtilt (double tjd,

                   double *mobl, double *tobl, double *eqeq,
                   double *psi, double *eps);

   void cel_pole (double del_dpsi, double del_deps);

   short int get_earth (double tjd, body *earth,

                        double *tdb, double *bary_earthp,
                        double *bary_earthv, double *helio_earthp,
                        double *helio_earthv);

   void proper_motion (double tjd1, double *pos1, double *vel1,
                       double tjd2,

                       double *pos2);

   void bary_to_geo (double *pos, double *earthvector,

                     double *pos2, double *lighttime);

   short int sun_field (double *pos, double *earthvector,

                        double *pos2);

   short int aberration (double *pos, double *vel, double lighttime,

                         double *pos2);

   void precession (double tjd1, double *pos, double tjd2,

                    double *pos2);

   short int nutate (double tjd, short int fn1, double *pos,

                     double *pos2);

   short int nutation_angles (double tdbtime,

                              double *longnutation,
                              double *obliqnutation);

   void fund_args (double t,

                   double a[5]);

   short int vector2radec (double *pos,

                           double *ra, double *dec);

   void radec2vector (double ra, double dec, double dist,

                      double *vector);

   void starvectors (cat_entry *star,

                     double *pos, double *vel);

   void tdb2tdt (double tdb,

                 double *tdtjd, double *secdiff);

   short int set_body (short int type, short int number, char *name,

                       body *cel_obj);

   short int ephemeris (double tjd, body *cel_obj, short int origin,

                        double *pos, double *vel);


   short int solarsystem (double tjd, short int body, short int origin, 

                          double *pos, double *vel);

   double *readeph (int mp, char *name, double jd,

                    int *err);

   void make_cat_entry (char catalog[4], char star_name[51],
                        long int star_num, double ra, double dec,
                        double pm_ra, double pm_dec, double parallax,
                        double rad_vel,

                        cat_entry *star);

   void transform_hip (cat_entry *hipparcos,

                       cat_entry *fk5);

   void transform_cat (short int option, double date_incat, 
                        cat_entry *incat, double date_newcat,
                        char newcat_id[4],

                        cat_entry *newcat);

   void equ2hor (double tjd, double deltat, double x, double y, 
                 site_info *location, double ra, double dec, 
                 short int ref_option,

                 double *zd, double *az, double *rar, double *decr);

   double refract (site_info *location, short int ref_option, 
                   double zd_obs);

   double julian_date (short int year, short int month, short int day,
                       double hour);

   void cal_date (double tjd,

                  short int *year, short int *month, short int *day,
                  double *hour);

//From Org solsys3.c
void sun_eph (double jd,

              double *ra, double *dec, double *dis);

//short int solarsystem (double tjd, short int body, short int origin, 
//
//                       double *pos, double *vel);



//From Org Novascon.
double PSI_COR;
double EPS_COR;
//Constants
//NOTE were "const"
    short int FN1;
    short int FN0;
    double T0;		//TDB Julian date of epoch J2000.0.
    double KMAU;		//Astronomical Unit in kilometers.
    double MAU;		//Astronomical Unit in meters.
    double C;		//Speed of light in AU/Day.
    double GS; 		//Heliocentric gravitational constant.
    double EARTHRAD; 	//Radius of Earth in kilometers.
    double F;  		//'f' = Earth ellipsoid flattening
    double OMEGA;  		//'omega' = rotational angular velocity of Earth in radians/sec
    double TWOPI;  		//Value of 2.0 * pi in radians.

   //Angle conversion constants.
    double RAD2SEC;
    double DEG2RAD;
    double RAD2DEG;



};

#endif // !defined(AFX_NOVAS_H__2B3CE12F_5F0D_11D3_AB4E_444553540000__INCLUDED_)
